# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/status/imu.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/status/imu.proto',
  package='blickfeld.protocol.status',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1a\x62lickfeld/status/imu.proto\x12\x19\x62lickfeld.protocol.status\x1a\x17\x62lickfeld/options.proto\"r\n\x03IMU\x12@\n\x0cstatic_state\x18\x01 \x01(\x0b\x32*.blickfeld.protocol.status.IMU.StaticState\x1a)\n\x0bStaticState\x12\x1a\n\x0c\x61\x63\x63\x65leration\x18\x01 \x03(\x02\x42\x04\xa0\xb5\x18\x06'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])




_IMU_STATICSTATE = _descriptor.Descriptor(
  name='StaticState',
  full_name='blickfeld.protocol.status.IMU.StaticState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='acceleration', full_name='blickfeld.protocol.status.IMU.StaticState.acceleration', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\240\265\030\006', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=196,
)

_IMU = _descriptor.Descriptor(
  name='IMU',
  full_name='blickfeld.protocol.status.IMU',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='static_state', full_name='blickfeld.protocol.status.IMU.static_state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_IMU_STATICSTATE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=82,
  serialized_end=196,
)

_IMU_STATICSTATE.containing_type = _IMU
_IMU.fields_by_name['static_state'].message_type = _IMU_STATICSTATE
DESCRIPTOR.message_types_by_name['IMU'] = _IMU
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

IMU = _reflection.GeneratedProtocolMessageType('IMU', (_message.Message,), {

  'StaticState' : _reflection.GeneratedProtocolMessageType('StaticState', (_message.Message,), {
    'DESCRIPTOR' : _IMU_STATICSTATE,
    '__module__' : 'blickfeld.status.imu_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.IMU.StaticState)
    })
  ,
  'DESCRIPTOR' : _IMU,
  '__module__' : 'blickfeld.status.imu_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.IMU)
  })
_sym_db.RegisterMessage(IMU)
_sym_db.RegisterMessage(IMU.StaticState)


_IMU_STATICSTATE.fields_by_name['acceleration']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/data/scanline.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2
from ..data import point_pb2 as blickfeld_dot_data_dot_point__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/data/scanline.proto',
  package='blickfeld.protocol.data',
  syntax='proto2',
  serialized_options=b'\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1d\x62lickfeld/data/scanline.proto\x12\x17\x62lickfeld.protocol.data\x1a\x17\x62lickfeld/options.proto\x1a\x1a\x62lickfeld/data/point.proto\"\x89\x01\n\x08Scanline\x12\x10\n\x02id\x18\x01 \x01(\x04\x42\x04\xb0\xb5\x18\x01\x12\x16\n\x08\x66rame_id\x18\x02 \x01(\x04\x42\x04\xb0\xb5\x18\x01\x12\x34\n\x06points\x18\x03 \x03(\x0b\x32\x1e.blickfeld.protocol.data.PointB\x04\xb0\xb5\x18\x01\x12\x1d\n\x0fstart_offset_ns\x18\x04 \x01(\x04\x42\x04\xb0\xb5\x18\x01\x42\x03\xf8\x01\x01'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_data_dot_point__pb2.DESCRIPTOR,])




_SCANLINE = _descriptor.Descriptor(
  name='Scanline',
  full_name='blickfeld.protocol.data.Scanline',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='blickfeld.protocol.data.Scanline.id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frame_id', full_name='blickfeld.protocol.data.Scanline.frame_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='points', full_name='blickfeld.protocol.data.Scanline.points', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_offset_ns', full_name='blickfeld.protocol.data.Scanline.start_offset_ns', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=249,
)

_SCANLINE.fields_by_name['points'].message_type = blickfeld_dot_data_dot_point__pb2._POINT
DESCRIPTOR.message_types_by_name['Scanline'] = _SCANLINE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Scanline = _reflection.GeneratedProtocolMessageType('Scanline', (_message.Message,), {
  'DESCRIPTOR' : _SCANLINE,
  '__module__' : 'blickfeld.data.scanline_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.Scanline)
  })
_sym_db.RegisterMessage(Scanline)


DESCRIPTOR._options = None
_SCANLINE.fields_by_name['id']._options = None
_SCANLINE.fields_by_name['frame_id']._options = None
_SCANLINE.fields_by_name['points']._options = None
_SCANLINE.fields_by_name['start_offset_ns']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/config/named_scan_pattern.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2
from ..config import scan_pattern_pb2 as blickfeld_dot_config_dot_scan__pattern__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/config/named_scan_pattern.proto',
  package='blickfeld.protocol.config',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)blickfeld/config/named_scan_pattern.proto\x12\x19\x62lickfeld.protocol.config\x1a\x17\x62lickfeld/options.proto\x1a#blickfeld/config/scan_pattern.proto\"\xa3\x01\n\x10NamedScanPattern\x12=\n\x04name\x18\x01 \x01(\tB/\xaa\xb5\x18+[a-zA-Z0-9]([a-zA-Z0-9]|[_ \\-][a-zA-Z0-9])*\x12\x36\n\x06\x63onfig\x18\x02 \x01(\x0b\x32&.blickfeld.protocol.config.ScanPattern\x12\x18\n\tread_only\x18\x03 \x01(\x08:\x05\x66\x61lse'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_config_dot_scan__pattern__pb2.DESCRIPTOR,])




_NAMEDSCANPATTERN = _descriptor.Descriptor(
  name='NamedScanPattern',
  full_name='blickfeld.protocol.config.NamedScanPattern',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='blickfeld.protocol.config.NamedScanPattern.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\252\265\030+[a-zA-Z0-9]([a-zA-Z0-9]|[_ \\-][a-zA-Z0-9])*', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config', full_name='blickfeld.protocol.config.NamedScanPattern.config', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='read_only', full_name='blickfeld.protocol.config.NamedScanPattern.read_only', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=298,
)

_NAMEDSCANPATTERN.fields_by_name['config'].message_type = blickfeld_dot_config_dot_scan__pattern__pb2._SCANPATTERN
DESCRIPTOR.message_types_by_name['NamedScanPattern'] = _NAMEDSCANPATTERN
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NamedScanPattern = _reflection.GeneratedProtocolMessageType('NamedScanPattern', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDSCANPATTERN,
  '__module__' : 'blickfeld.config.named_scan_pattern_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.NamedScanPattern)
  })
_sym_db.RegisterMessage(NamedScanPattern)


_NAMEDSCANPATTERN.fields_by_name['name']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/status/server.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2
from ..stream import connection_pb2 as blickfeld_dot_stream_dot_connection__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/status/server.proto',
  package='blickfeld.protocol.status',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1d\x62lickfeld/status/server.proto\x12\x19\x62lickfeld.protocol.status\x1a\x17\x62lickfeld/options.proto\x1a!blickfeld/stream/connection.proto\"\xdb\x04\n\x06Server\x12\x39\n\x07\x63lients\x18\x01 \x03(\x0b\x32(.blickfeld.protocol.status.Server.Client\x12\x45\n\rnetwork_stats\x18\x02 \x01(\x0b\x32..blickfeld.protocol.status.Server.NetworkStats\x1a\xab\x02\n\x0cNetworkStats\x12\x44\n\x04sent\x18\x01 \x01(\x0b\x32\x36.blickfeld.protocol.status.Server.NetworkStats.Channel\x12H\n\x08received\x18\x02 \x01(\x0b\x32\x36.blickfeld.protocol.status.Server.NetworkStats.Channel\x12\x1b\n\x10\x64ropped_messages\x18\x03 \x01(\x04:\x01\x30\x1an\n\x07\x43hannel\x12\x1b\n\x10total_byte_count\x18\x01 \x01(\x04:\x01\x30\x12\x1e\n\x10\x62ytes_per_second\x18\x02 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x12&\n\x18maximum_bytes_per_second\x18\x03 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x1a\xa0\x01\n\x06\x43lient\x12;\n\rsubscriptions\x18\x01 \x03(\x0b\x32$.blickfeld.protocol.stream.Subscribe\x12\x45\n\rnetwork_stats\x18\x02 \x01(\x0b\x32..blickfeld.protocol.status.Server.NetworkStats\x12\x12\n\nidentifier\x18\x03 \x01(\t'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_stream_dot_connection__pb2.DESCRIPTOR,])




_SERVER_NETWORKSTATS_CHANNEL = _descriptor.Descriptor(
  name='Channel',
  full_name='blickfeld.protocol.status.Server.NetworkStats.Channel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='total_byte_count', full_name='blickfeld.protocol.status.Server.NetworkStats.Channel.total_byte_count', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes_per_second', full_name='blickfeld.protocol.status.Server.NetworkStats.Channel.bytes_per_second', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='maximum_bytes_per_second', full_name='blickfeld.protocol.status.Server.NetworkStats.Channel.maximum_bytes_per_second', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=451,
  serialized_end=561,
)

_SERVER_NETWORKSTATS = _descriptor.Descriptor(
  name='NetworkStats',
  full_name='blickfeld.protocol.status.Server.NetworkStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sent', full_name='blickfeld.protocol.status.Server.NetworkStats.sent', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='received', full_name='blickfeld.protocol.status.Server.NetworkStats.received', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dropped_messages', full_name='blickfeld.protocol.status.Server.NetworkStats.dropped_messages', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SERVER_NETWORKSTATS_CHANNEL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=262,
  serialized_end=561,
)

_SERVER_CLIENT = _descriptor.Descriptor(
  name='Client',
  full_name='blickfeld.protocol.status.Server.Client',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='subscriptions', full_name='blickfeld.protocol.status.Server.Client.subscriptions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='network_stats', full_name='blickfeld.protocol.status.Server.Client.network_stats', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='identifier', full_name='blickfeld.protocol.status.Server.Client.identifier', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=564,
  serialized_end=724,
)

_SERVER = _descriptor.Descriptor(
  name='Server',
  full_name='blickfeld.protocol.status.Server',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='clients', full_name='blickfeld.protocol.status.Server.clients', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='network_stats', full_name='blickfeld.protocol.status.Server.network_stats', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SERVER_NETWORKSTATS, _SERVER_CLIENT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=724,
)

_SERVER_NETWORKSTATS_CHANNEL.containing_type = _SERVER_NETWORKSTATS
_SERVER_NETWORKSTATS.fields_by_name['sent'].message_type = _SERVER_NETWORKSTATS_CHANNEL
_SERVER_NETWORKSTATS.fields_by_name['received'].message_type = _SERVER_NETWORKSTATS_CHANNEL
_SERVER_NETWORKSTATS.containing_type = _SERVER
_SERVER_CLIENT.fields_by_name['subscriptions'].message_type = blickfeld_dot_stream_dot_connection__pb2._SUBSCRIBE
_SERVER_CLIENT.fields_by_name['network_stats'].message_type = _SERVER_NETWORKSTATS
_SERVER_CLIENT.containing_type = _SERVER
_SERVER.fields_by_name['clients'].message_type = _SERVER_CLIENT
_SERVER.fields_by_name['network_stats'].message_type = _SERVER_NETWORKSTATS
DESCRIPTOR.message_types_by_name['Server'] = _SERVER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Server = _reflection.GeneratedProtocolMessageType('Server', (_message.Message,), {

  'NetworkStats' : _reflection.GeneratedProtocolMessageType('NetworkStats', (_message.Message,), {

    'Channel' : _reflection.GeneratedProtocolMessageType('Channel', (_message.Message,), {
      'DESCRIPTOR' : _SERVER_NETWORKSTATS_CHANNEL,
      '__module__' : 'blickfeld.status.server_pb2'
      # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.Server.NetworkStats.Channel)
      })
    ,
    'DESCRIPTOR' : _SERVER_NETWORKSTATS,
    '__module__' : 'blickfeld.status.server_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.Server.NetworkStats)
    })
  ,

  'Client' : _reflection.GeneratedProtocolMessageType('Client', (_message.Message,), {
    'DESCRIPTOR' : _SERVER_CLIENT,
    '__module__' : 'blickfeld.status.server_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.Server.Client)
    })
  ,
  'DESCRIPTOR' : _SERVER,
  '__module__' : 'blickfeld.status.server_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.Server)
  })
_sym_db.RegisterMessage(Server)
_sym_db.RegisterMessage(Server.NetworkStats)
_sym_db.RegisterMessage(Server.NetworkStats.Channel)
_sym_db.RegisterMessage(Server.Client)


_SERVER_NETWORKSTATS_CHANNEL.fields_by_name['bytes_per_second']._options = None
_SERVER_NETWORKSTATS_CHANNEL.fields_by_name['maximum_bytes_per_second']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/config/product.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/config/product.proto',
  package='blickfeld.protocol.config',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1e\x62lickfeld/config/product.proto\x12\x19\x62lickfeld.protocol.config*3\n\x07Product\x12\x10\n\x0cPRODUCT_CUBE\x10\x00\x12\x16\n\x12PRODUCT_CUBE_RANGE\x10\x02'
)

_PRODUCT = _descriptor.EnumDescriptor(
  name='Product',
  full_name='blickfeld.protocol.config.Product',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PRODUCT_CUBE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRODUCT_CUBE_RANGE', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=61,
  serialized_end=112,
)
_sym_db.RegisterEnumDescriptor(_PRODUCT)

Product = enum_type_wrapper.EnumTypeWrapper(_PRODUCT)
PRODUCT_CUBE = 0
PRODUCT_CUBE_RANGE = 2


DESCRIPTOR.enum_types_by_name['Product'] = _PRODUCT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/status/temperature.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/status/temperature.proto',
  package='blickfeld.protocol.status',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"blickfeld/status/temperature.proto\x12\x19\x62lickfeld.protocol.status\x1a\x17\x62lickfeld/options.proto\"\xe9\x01\n\x0bTemperature\x12=\n\x06sensor\x18\x01 \x01(\x0e\x32-.blickfeld.protocol.status.Temperature.Sensor\x12\x0f\n\x05value\x18\x02 \x01(\x02H\x00\x12\x17\n\rfailed_reason\x18\x03 \x01(\tH\x00\"i\n\x06Sensor\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03LDM\x10\x01\x12\x0c\n\x08\x45THERNET\x10\x02\x12\x07\n\x03MSB\x10\x03\x12\x06\n\x02PL\x10\x04\x12\x06\n\x02PS\x10\x05\x12\x11\n\rSCANNER_SMALL\x10\x06\x12\x0f\n\x0bSCANNER_BIG\x10\x07\x42\x06\n\x04\x64\x61ta'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])



_TEMPERATURE_SENSOR = _descriptor.EnumDescriptor(
  name='Sensor',
  full_name='blickfeld.protocol.status.Temperature.Sensor',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LDM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ETHERNET', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MSB', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PL', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PS', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SCANNER_SMALL', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SCANNER_BIG', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=211,
  serialized_end=316,
)
_sym_db.RegisterEnumDescriptor(_TEMPERATURE_SENSOR)


_TEMPERATURE = _descriptor.Descriptor(
  name='Temperature',
  full_name='blickfeld.protocol.status.Temperature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sensor', full_name='blickfeld.protocol.status.Temperature.sensor', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='blickfeld.protocol.status.Temperature.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failed_reason', full_name='blickfeld.protocol.status.Temperature.failed_reason', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TEMPERATURE_SENSOR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='blickfeld.protocol.status.Temperature.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=91,
  serialized_end=324,
)

_TEMPERATURE.fields_by_name['sensor'].enum_type = _TEMPERATURE_SENSOR
_TEMPERATURE_SENSOR.containing_type = _TEMPERATURE
_TEMPERATURE.oneofs_by_name['data'].fields.append(
  _TEMPERATURE.fields_by_name['value'])
_TEMPERATURE.fields_by_name['value'].containing_oneof = _TEMPERATURE.oneofs_by_name['data']
_TEMPERATURE.oneofs_by_name['data'].fields.append(
  _TEMPERATURE.fields_by_name['failed_reason'])
_TEMPERATURE.fields_by_name['failed_reason'].containing_oneof = _TEMPERATURE.oneofs_by_name['data']
DESCRIPTOR.message_types_by_name['Temperature'] = _TEMPERATURE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Temperature = _reflection.GeneratedProtocolMessageType('Temperature', (_message.Message,), {
  'DESCRIPTOR' : _TEMPERATURE,
  '__module__' : 'blickfeld.status.temperature_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.status.Temperature)
  })
_sym_db.RegisterMessage(Temperature)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/config/algorithm.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/config/algorithm.proto',
  package='blickfeld.protocol.config',
  syntax='proto2',
  serialized_options=b'\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n blickfeld/config/algorithm.proto\x12\x19\x62lickfeld.protocol.config\x1a\x17\x62lickfeld/options.proto\"\xcf\x04\n\tAlgorithm\x12\\\n\x16\x62\x61\x63kground_subtraction\x18\x01 \x01(\x0b\x32:.blickfeld.protocol.config.Algorithm.BackgroundSubtractionH\x00\x12N\n\x0fneighbor_filter\x18\x02 \x01(\x0b\x32\x33.blickfeld.protocol.config.Algorithm.NeighborFilterH\x00\x12Z\n\x15static_transformation\x18\x03 \x01(\x0b\x32\x39.blickfeld.protocol.config.Algorithm.StaticTransformationH\x00\x1a\xb8\x01\n\x15\x42\x61\x63kgroundSubtraction\x12\x39\n\x15initialization_frames\x18\x01 \x01(\r:\x02\x31\x30\x42\x16\x81\xb5\x18\x00\x00\x00\x00\x00\x00\xf0?\x89\xb5\x18\x00\x00\x00\x00\x00\x00$@\x12>\n\x16\x65xponential_decay_rate\x18\x02 \x01(\x02:\x06\x30.0025B\x16\x81\xb5\x18\x00\x00\x00\x00\x00\x00\x00\x00\x89\xb5\x18\x00\x00\x00\x00\x00\x00\xf0?J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\x18\x63lassification_threshold\x1a\x10\n\x0eNeighborFilter\x1a\x63\n\x14StaticTransformation\x12#\n\x0frotation_matrix\x18\x01 \x03(\x02\x42\n\x10\x01\xa0\xb5\x18\x12\xb0\xb5\x18\x01\x12&\n\x12translation_vector\x18\x02 \x03(\x02\x42\n\x10\x01\xa0\xb5\x18\x06\xb0\xb5\x18\x01\x42\x06\n\x04\x61lgoB\x03\xf8\x01\x01'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])




_ALGORITHM_BACKGROUNDSUBTRACTION = _descriptor.Descriptor(
  name='BackgroundSubtraction',
  full_name='blickfeld.protocol.config.Algorithm.BackgroundSubtraction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initialization_frames', full_name='blickfeld.protocol.config.Algorithm.BackgroundSubtraction.initialization_frames', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=10,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\201\265\030\000\000\000\000\000\000\360?\211\265\030\000\000\000\000\000\000$@', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exponential_decay_rate', full_name='blickfeld.protocol.config.Algorithm.BackgroundSubtraction.exponential_decay_rate', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.0025),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\201\265\030\000\000\000\000\000\000\000\000\211\265\030\000\000\000\000\000\000\360?', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=369,
  serialized_end=553,
)

_ALGORITHM_NEIGHBORFILTER = _descriptor.Descriptor(
  name='NeighborFilter',
  full_name='blickfeld.protocol.config.Algorithm.NeighborFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=555,
  serialized_end=571,
)

_ALGORITHM_STATICTRANSFORMATION = _descriptor.Descriptor(
  name='StaticTransformation',
  full_name='blickfeld.protocol.config.Algorithm.StaticTransformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rotation_matrix', full_name='blickfeld.protocol.config.Algorithm.StaticTransformation.rotation_matrix', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001\240\265\030\022\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='translation_vector', full_name='blickfeld.protocol.config.Algorithm.StaticTransformation.translation_vector', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001\240\265\030\006\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=573,
  serialized_end=672,
)

_ALGORITHM = _descriptor.Descriptor(
  name='Algorithm',
  full_name='blickfeld.protocol.config.Algorithm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='background_subtraction', full_name='blickfeld.protocol.config.Algorithm.background_subtraction', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='neighbor_filter', full_name='blickfeld.protocol.config.Algorithm.neighbor_filter', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_transformation', full_name='blickfeld.protocol.config.Algorithm.static_transformation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ALGORITHM_BACKGROUNDSUBTRACTION, _ALGORITHM_NEIGHBORFILTER, _ALGORITHM_STATICTRANSFORMATION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='algo', full_name='blickfeld.protocol.config.Algorithm.algo',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=89,
  serialized_end=680,
)

_ALGORITHM_BACKGROUNDSUBTRACTION.containing_type = _ALGORITHM
_ALGORITHM_NEIGHBORFILTER.containing_type = _ALGORITHM
_ALGORITHM_STATICTRANSFORMATION.containing_type = _ALGORITHM
_ALGORITHM.fields_by_name['background_subtraction'].message_type = _ALGORITHM_BACKGROUNDSUBTRACTION
_ALGORITHM.fields_by_name['neighbor_filter'].message_type = _ALGORITHM_NEIGHBORFILTER
_ALGORITHM.fields_by_name['static_transformation'].message_type = _ALGORITHM_STATICTRANSFORMATION
_ALGORITHM.oneofs_by_name['algo'].fields.append(
  _ALGORITHM.fields_by_name['background_subtraction'])
_ALGORITHM.fields_by_name['background_subtraction'].containing_oneof = _ALGORITHM.oneofs_by_name['algo']
_ALGORITHM.oneofs_by_name['algo'].fields.append(
  _ALGORITHM.fields_by_name['neighbor_filter'])
_ALGORITHM.fields_by_name['neighbor_filter'].containing_oneof = _ALGORITHM.oneofs_by_name['algo']
_ALGORITHM.oneofs_by_name['algo'].fields.append(
  _ALGORITHM.fields_by_name['static_transformation'])
_ALGORITHM.fields_by_name['static_transformation'].containing_oneof = _ALGORITHM.oneofs_by_name['algo']
DESCRIPTOR.message_types_by_name['Algorithm'] = _ALGORITHM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Algorithm = _reflection.GeneratedProtocolMessageType('Algorithm', (_message.Message,), {

  'BackgroundSubtraction' : _reflection.GeneratedProtocolMessageType('BackgroundSubtraction', (_message.Message,), {
    'DESCRIPTOR' : _ALGORITHM_BACKGROUNDSUBTRACTION,
    '__module__' : 'blickfeld.config.algorithm_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.Algorithm.BackgroundSubtraction)
    })
  ,

  'NeighborFilter' : _reflection.GeneratedProtocolMessageType('NeighborFilter', (_message.Message,), {
    'DESCRIPTOR' : _ALGORITHM_NEIGHBORFILTER,
    '__module__' : 'blickfeld.config.algorithm_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.Algorithm.NeighborFilter)
    })
  ,

  'StaticTransformation' : _reflection.GeneratedProtocolMessageType('StaticTransformation', (_message.Message,), {
    'DESCRIPTOR' : _ALGORITHM_STATICTRANSFORMATION,
    '__module__' : 'blickfeld.config.algorithm_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.Algorithm.StaticTransformation)
    })
  ,
  'DESCRIPTOR' : _ALGORITHM,
  '__module__' : 'blickfeld.config.algorithm_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.Algorithm)
  })
_sym_db.RegisterMessage(Algorithm)
_sym_db.RegisterMessage(Algorithm.BackgroundSubtraction)
_sym_db.RegisterMessage(Algorithm.NeighborFilter)
_sym_db.RegisterMessage(Algorithm.StaticTransformation)


DESCRIPTOR._options = None
_ALGORITHM_BACKGROUNDSUBTRACTION.fields_by_name['initialization_frames']._options = None
_ALGORITHM_BACKGROUNDSUBTRACTION.fields_by_name['exponential_decay_rate']._options = None
_ALGORITHM_STATICTRANSFORMATION.fields_by_name['rotation_matrix']._options = None
_ALGORITHM_STATICTRANSFORMATION.fields_by_name['translation_vector']._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/python
#
# Copyright (c) 2020 Blickfeld GmbH.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE.md file in the root directory of this source tree.
#

from __future__ import print_function

import logging
import re
import socket
import struct
import ssl
import tempfile
import os
import warnings
import time
from distutils.version import LooseVersion, StrictVersion

from .protocol import connection_pb2, error_pb2
from .protocol.stream import connection_pb2 as stream_connection_pb2
from .protocol import options_pb2, common_pb2
from .protocol.config import algorithm_pb2
from .protocol.status import time_synchronization_pb2
from . import stream
from .version import __version__

log = logging.getLogger(__name__)

class scanner(object):
    """
    Scanner is a class to connect to a device, get a point cloud stream, get a status stream and set a scan pattern.

    :param hostname_or_ip: Device to which the connection should be established.
    :type hostname_or_ip: str
    :param port: Port on which the device is reachable the default is 8000.
    :type port: int
    :param timeout: Timeout in seconds until connection is aborted
    :type timeout: int
    :param name: Name of the device used for string representation.
    :type name: str
    :param key_and_cert_file: Filename containing a private key and certificate for SSL connection.
    :type key_and_cert_file: str
    :param key_and_cert: String containing a private key and certificate for SSL connection.
    :type key_and_cert: str
    """
    protocol_version = 1
    _default_port = 8000
    _default_ssl_port = 8800
    
    def __init__(self, hostname_or_ip="localhost", port=None, timeout=30, name=None, key_and_cert_file=None, key_and_cert=None):
        self.hostname_or_ip = hostname_or_ip
        self.timeout = timeout
        self.name = name if name is not None else self.hostname_or_ip
        self.api_path = "http://" + hostname_or_ip + "/api/v1/"
        
        self._key_and_cert = None
        if key_and_cert_file:
            with open(key_and_cert_file,'r') as f:
                self._key_and_cert = f.read()
        elif key_and_cert:
            self._key_and_cert = key_and_cert
        self.has_ssl = self._key_and_cert != None

        if port != None:
            self.port = port
        else:
            self.port = self._default_ssl_port if self.has_ssl else self._default_port
        
        self._connection = self.create_connection()
        
        self._hello = self.hello()
        # Compare library version of server and client
        if __version__ != self._hello.library_version:
            def parse_version(version_str):
                return [int(x) for x in version_str.split(".")]
            
            local_version = None
            if self._hello.library_version and __version__:
                local_version = [int(x) if x.isdecimal() else None for x in __version__.split(".")]
                server_version = [int(x) if x.isdecimal() else None for x in self._hello.library_version.split(".")]
            # Only warn if major or minor version does not match
            if local_version is None or len(local_version) < 3 or len(server_version) < 3 or local_version[0] != server_version[0] or local_version[1] != server_version[1]:
                warnings.warn("Warning: The client BSL version does not match the server BSL version. Client has '%s', server has '%s'." % (__version__, self._hello.library_version), DeprecationWarning, stacklevel=2)

    def __del__(self):
        if hasattr(self, "_connection"):
            self._connection.close()
            
    def __repr__(self):
        return "<Blickfeld Scanner '%s'>" % ( self.name )

    def hello(self):
        """ Send hello to device and retrieve device, software, and hardware information

        :returns: Hello object, see :any:`protobuf_protocol` Response.Hello
        """
        req = connection_pb2.Request()
        req.hello.protocol_version = self.protocol_version
        req.hello.library_version = __version__
        req.hello.language = common_pb2.Language.PYTHON
        return self._connection.send_request(req).hello

    def serial_number(self):
        """ > Introduced in BSL v2.16 and firmware v1.17

        Retrieve unique serial number of the device.
        Consider using hello() to retrieve more information.

        :returns: Unique serial number of the device
        """
        hello = self.hello()
        return hello.serial_number if hello.HasField("serial_number") else hello.legacy_serial_number

    def product(self):
        """ > Introduced in BSL v2.16 and firmware v1.17

        Retrieve product type.
        Consider using hello() to retrieve more information.

        :returns: Product type of the device, see :any:`protobuf_protocol` config.Product
        """
        return self.hello().product

    def get_status(self):
        """ Request status of device

        :returns: Status object, see :any:`protobuf_protocol` Status
        """
        req = connection_pb2.Request()
        req.status.SetInParent()
        return self._connection.send_request(req).status

    def get_status_stream(self):
        """ Request status stream of device

        This returns a status stream, which only delivers updates when the device has changed.
        Do not use this in a synchronous blocking call. Use the `get_status` method instead.

        :returns: :py:class:`blickfeld_scanner.scanner.stream.status` object
        """
        return stream.status(self.create_connection())

    def get_point_cloud_stream(self, filter=None, reference_frame=None, point_filter=None, algorithms=None, as_numpy=False):
        """ Request point cloud stream of device

        :param filter: DEPRECATED

            > Introduced in BSL v2.10 and firmware v1.9

            Filter points and returns by point attributes during the post-processing on the device. Is replaced by 'point_filter'.
        :param reference_frame:
            > Introduced in BSL v2.10 and firmware v1.9

            Frame representing the desired data. To request a field, set it to any value (also in submessages). For a repeated field, add at least one element.
        :param point_filter:
            > Introduced in BSL v2.13 and firmware v1.9

            Filter points and returns by point attributes during the post-processing on the device. This replaces the 'filter' parameter
        :param algorithms:
            > Introduced in BSL v2.17 and firmware v1.16

            Add set of algorithms which post-process the point cloud data.
        :param as_numpy:
            > Introduced in BSL v2.16
    
            This enables numpy support of the point cloud stream. Use recv_frame_as_numpy() to fetch frames as numpy structured arrays.
            Specify the required attributes via the reference_frame attribute.
            
            Note: With devices with an firmware version older than v1.17, the performance is significantly worse than with up-to-date devices as the format needs to be converted on the client side. Consider updating the device.
    
        :returns: :py:class:`blickfeld_scanner.scanner.stream.point_cloud` object
        """
        if filter and point_filter:
            raise Exception("Either provide 'filter' or 'point_filter'.\nPlease use 'point_filter' instead of 'filter'. 'filter' is deprecated.")
        if filter:
            warnings.warn("Please use 'point_filter' instead of 'filter'. 'filter' is deprecated.", DeprecationWarning, stacklevel=2)
            point_filter = filter
        return stream.point_cloud(self.create_connection(), point_filter=point_filter, reference_frame=reference_frame, algorithms=algorithms, as_numpy=as_numpy)

    def get_raw_point_cloud_stream(self, point_filter=None, reference_frame=None):
        """ Request raw point cloud stream of device
        
        > Introduced in BSL v2.13 and firmware v1.13

        :param point_filter:
            > Introduced in BSL v2.10 and firmware v1.9

            Filter points and returns by point attributes during the post-processing on the device.
        :param reference_frame:
            > Introduced in BSL v2.10 and firmware v1.9

            Frame representing the desired data. To request a field, set it to any value (also in submessages). For a repeated field, add at least one element.
        :returns: :py:class:`blickfeld_scanner.scanner.stream.raw` object
        """
        req = stream_connection_pb2.Subscribe.RawFile()
        req.point_cloud.SetInParent()
        if point_filter:
            req.point_cloud.filter.CopyFrom(point_filter)
        if reference_frame:
            req.point_cloud.reference_frame.CopyFrom(reference_frame)

        return stream.raw(self.create_connection(), req)

    def record_point_cloud_stream(self, file_name, point_filter=None, reference_frame=None):
        """ Record point cloud stream to file
        
        > Introduced in BSL v2.13 and firmware v1.13
        
        :param file_name: Path to the file where it should be dumped

        :param point_filter:
            > Introduced in BSL v2.10 and firmware v1.9

            Filter points and returns by point attributes during the post-processing on the device.
        :param reference_frame:
            > Introduced in BSL v2.10 and firmware v1.9

            Frame representing the desired data. To request a field, set it to any value (also in submessages). For a repeated field, add at least one element.
        :returns: :py:class:`blickfeld_scanner.scanner.stream.raw` object
        """
        req = stream_connection_pb2.Subscribe.RawFile()
        req.point_cloud.SetInParent()
        if point_filter:
            req.point_cloud.filter.CopyFrom(point_filter)
        if reference_frame:
            req.point_cloud.reference_frame.CopyFrom(reference_frame)

        return stream.raw(self.create_connection(), req, file_name)
        
    @staticmethod
    def file_point_cloud_stream(dump_filename, as_numpy=False):
        """ Request a point_cloud_stream, which streams off a .bfpc file.
        No device (and connection to a device) is needed for this operation.

        :param dump_filename: path to .bfpc file
        :param as_numpy:
            > Introduced in BSL v2.16

            This enables numpy support of the point cloud stream. Use recv_frame_as_numpy() to fetch frames as numpy structured arrays.
            Specify the required attributes via the reference_frame attribute.

            Note: The performance of this option is significantly better with files that were recorded in the packed format. Packed recordings are available in firmware versions >= v1.17.
        :return: :py:class:`blickfeld_scanner.scanner.stream.point_cloud` object
        """
        return stream.point_cloud(from_file=dump_filename, as_numpy=as_numpy)

    def set_default_point_cloud_algorithms(self, persist=False, background_subtraction=False, neighbor_filter=False, static_transformation=False):
        """ Sets point cloud algorithms, which are then applied on all point cloud stream subscribe requests of clients.
        Clients can still actively request other algorithms or disabled them.

        Args:
            persist (bool, optional): Persist the configuration on the device and activates it after a power-cycle. Defaults to False.
            background_subtraction (bool, optional): Either enable the background subtraction with default parameters or supply a configuration object. Defaults to False.
            neighbor_filter (bool, optional): Either enable the neighbor filter with default parameters or supply a configuration object Defaults to False.
            static_transformation (bool, optional): Either enable the static transformation with default parameters or supply a configuration object. Defaults to False.

        Returns:
            Currently set advanced config, see :any:`protobuf_protocol` advanced config
        """
        advanced = self.get_advanced_config()
        del advanced.server.default_point_cloud_subscription.algorithms[:]
        
        algorithms = []
        if background_subtraction:
            cfg = algorithm_pb2.Algorithm()
            if type(background_subtraction) == bool:
                cfg.background_subtraction.SetInParent()
            else:
                cfg.background_subtraction.CopyFrom(background_subtraction)
            algorithms.append(cfg)
        if neighbor_filter:
            cfg = algorithm_pb2.Algorithm()
            if type(neighbor_filter) == bool:
                cfg.neighbor_filter.SetInParent()
            else:
                cfg.neighbor_filter.CopyFrom(neighbor_filter)
            algorithms.append(cfg)
        if static_transformation:
            cfg = algorithm_pb2.Algorithm()
            if type(static_transformation) == bool:
                cfg.static_transformation.SetInParent()
            else:
                cfg.static_transformation.CopyFrom(static_transformation)
            algorithms.append(cfg)
        
        advanced.server.default_point_cloud_subscription.algorithms.extend(algorithms)
        return self.set_advanced_config(advanced, persist=persist)
    
    def set_scan_pattern(self, config=None, name=None, persist = False):
        """ Function to set a new scan pattern, see: :any:`protobuf_protocol`.
        First call :py:func:`blickfeld_scanner.scanner.scanner.fill_scan_pattern` with the scan pattern you want to set and
        then use that returned scan pattern as a config in this function.

        > Changed in BSL v2.15 and firmware v1.16

        It is now possible to set a named scan pattern. Either a scan pattern config or a name of a named scan pattern can be provided.

        :param config: scan pattern to be set
        :param name: name of named scan pattern to be set
        :param persist: Persist scan pattern on device and reload it after a power-cycle, see: :any:`protobuf_protocol` scan pattern
        :return: response scan pattern, see :any:`protobuf_protocol` Connection, see :any:`protobuf_protocol` Connection
        """
        req = connection_pb2.Request()
        if name and config:
            raise ValueError("Provide either config or named scan pattern name.")
        if name:
            req.set_scan_pattern.name = name
        else:
            req.set_scan_pattern.config.MergeFrom(config)
        req.set_scan_pattern.persist = persist
        return self._connection.send_request(req).set_scan_pattern
    
    def fill_scan_pattern(self, config):
        """ Function to fill a scan pattern, see: :any:`protobuf_protocol`. The device fill the missing fields with default values and return it. After it is filled the scan pattern can be set with the :py:func:`blickfeld_scanner.scanner.scanner.set_scan_pattern` function.

        :param config: scan pattern to be filled by the BSS, see: :any:`protobuf_protocol` scan pattern
        :return: Filled scan pattern, which can be set, see :any:`protobuf_protocol` scan pattern
        """
        req = connection_pb2.Request()
        req.fill_scan_pattern.config.MergeFrom(config)
        return self._connection.send_request(req).fill_scan_pattern.config
    
    def get_scan_pattern(self):
        """ Returns the currently set scan pattern, see: :any:`protobuf_protocol`.

        :return: Currently set scan pattern, see :any:`protobuf_protocol` scan pattern
        """
        req = connection_pb2.Request()
        req.get_scan_pattern.SetInParent()
        return self._connection.send_request(req).get_scan_pattern.config
    
    def get_scan_pattern_constraints(self):
        """This request returns a list of constraints which are applied on scan patterns.
        
        > Introduced in BSL v2.14 and firmware v1.14
        
        The constraints define the constant and dynamic relationships between field values.
        The constraints are equal for a device type and firmware, but might vary for firmware releases and device variants.
        It is mainly used to visualize the constraints in the scan pattern configuration of the web gui.

        :return: List of active constraints, see :any:`protobuf_protocol` common
        """
        req = connection_pb2.Request()
        req.get_scan_pattern_constraints.SetInParent()
        return self._connection.send_request(req).get_scan_pattern_constraints.constraints
    
    def set_advanced_config(self, config, persist = False):
        """ Function to set advanced config, see: :any:`protobuf_protocol`.
        Expert parameters: It is not recommended to adapt this calibrated configuration without understanding the influences on the resulting point cloud quality.

        > Introduced in BSL v2.11 and firmware v1.11

        :param config: advanced config to be set
        :param persist: Persist advanced config on device and reload it after a power-cycle
        :return: response advanced config, see :any:`protobuf_protocol` advanced config
        """
        req = connection_pb2.Request()
        req.set_advanced_config.config.MergeFrom(config)
        req.set_advanced_config.persist = persist
        return self._connection.send_request(req).set_advanced_config
    
    def get_advanced_config(self):
        """ Returns the currently set advanced config, see: :any:`protobuf_protocol`.
        
        > Introduced in BSL v2.11 and firmware v1.11

        :return: Currently set advanced config, see :any:`protobuf_protocol` advanced config
        """
        req = connection_pb2.Request()
        req.get_advanced_config.SetInParent()
        return self._connection.send_request(req).get_advanced_config.config
    
    def get_device_timestamp(self):
        """Returns the current device timestamp in seconds.

        :return: Current device timestamp in seconds
        """
        req = connection_pb2.Request()
        req.hello.protocol_version = self.protocol_version
        return self._connection.send_request(req).timestamp_ns / 1e9

    def set_time_synchronization(self, ntp=False, ptp=False, persist=True, wait_for_sync=True, max_sync_duration=60):
        """> Introduced in BSL v2.18 and firmware v1.19
        
        Set the type of time synchronization and set basic parameters

        Either NTP or PTP has to be set (If both are set NTP will be choosen). It will overwrite the current time synchronization configuration

        :param ntp: Set this field to choose NTPv4 time synchronization, this can be a list of ntp servers or a bool if the standard config should be used
        :param ptp: Set this field to choose PTP time synchronization, this can be a list of ptp unicast destinations, this will also activate unicast mode and deactivate multicast mode or a bool if the standard config should be used
        :param persist: Persist time synchronization config on device and reload it after a power-cycle
        :param wait_for_sync: Wait until device is synchronized. Raises exception if device is not synchronized within max_sync_duration.
        :param max_sync_duration: Specify maximum time in seconds for synchronization.
        :return: None.
        """
        cfg = self.get_advanced_config()

        if ntp and ptp:
            raise AttributeError("Either provide ntp or ptp configuration. Not both.")
        if not ntp and not ptp:
            raise AttributeError("Neither ntp nor ptp is provided.")

        if ntp:
            if type(ntp) == bool:
                cfg.time_synchronization.ntp.SetInParent()
            elif type(ntp) == list:
                cfg.time_synchronization.ntp.servers[:] = ntp
            else:
                cfg.time_synchronization.ntp.CopyFrom(ntp)
        if ptp:
            if type(ptp) == bool:
                cfg.time_synchronization.ptp.SetInParent()
            elif type(ptp) == list:
                cfg.time_synchronization.ptp.unicast_destinations[:] = ptp
            else:
                cfg.time_synchronization.ptp.CopyFrom(ptp)

        ret = self.set_advanced_config(cfg, persist=persist)

        if wait_for_sync:
            while max_sync_duration > 0:
                status = self.get_status()
                if status.time_synchronization.state == time_synchronization_pb2.TimeSynchronization.State.SYNCED:
                    return
                time.sleep(1)
                max_sync_duration = max_sync_duration - 1
        
            raise Exception("Device failed to synchronize. Current state is %s." % (time_synchronization_pb2.TimeSynchronization.State.Name(status.time_synchronization.state)))
            
    def run_self_test(self):
        """> Introduced in BSL v2.10 and firmware v1.9
        
        Run self test on the device.
        See :any:`protobuf_protocol` Request.RunSelfTest.

        :return: Self test run response, see :any:`protobuf_protocol` Response.RunSelfTest
        """
        req = connection_pb2.Request()
        req.run_self_test.SetInParent()
        return self._connection.send_request(req).run_self_test
        
    def create_connection(self):
        """Function to create a new connection
        
        :return: Newly created :py:class:`blickfeld_scanner.scanner.connection`
        """
        return connection(self.hostname_or_ip, self.port, timeout=self.timeout, key_and_cert=self._key_and_cert)
            
    def attempt_error_recovery(self):
        """> Introduced in BSL v2.13 and firmware v1.13
        
        This request can be used to attempt a re-initialization of the device if it is errored.
        A self test is automatically triggered after a successful re-initialization.
        """
        req = connection_pb2.Request()
        req.attempt_error_recovery.SetInParent()
        return self._connection.send_request(req).attempt_error_recovery
    
    def get_named_scan_patterns(self):
        """> Introduced in BSL v2.15 and firmware v1.16
        
        Get protobuf list of named scan patterns. There are two types of named scan patterns:
        1. Default scan patterns, which are not changeable.
        2. User defined named scan patterns, which are changeable.

        :return: List of named scan patterns, see :any:`protobuf_protocol` Response.GetNamedScanPatterns
        """
        req = connection_pb2.Request()
        req.get_named_scan_patterns.SetInParent()
        return self._connection.send_request(req).get_named_scan_patterns
    
    def store_named_scan_pattern(self, name, config):
        """> Introduced in BSL v2.15 and firmware v1.16
        
        Store a named scan patterns.
        The default scan patterns can't be overwritten.

        :param name: Name of the scan pattern
        :param config: Config of the scan pattern, see :any:`protobuf_protocol` ScanPattern
        """
        req = connection_pb2.Request()
        req.store_named_scan_pattern.name = name
        req.store_named_scan_pattern.config.CopyFrom(config)
        return self._connection.send_request(req).store_named_scan_pattern
    
    def delete_named_scan_pattern(self, name):
        """> Introduced in BSL v2.15 and firmware v1.16

        Delete a named scan patterns.
        The default scan patterns can't be deleted.

        :param name: Name of the scan pattern
        """
        req = connection_pb2.Request()
        req.delete_named_scan_pattern.name = name
        return self._connection.send_request(req).delete_named_scan_pattern

    def get_imu_stream(self, as_numpy=False):
        """> Introduced in BSL v2.18 and firmware v1.19
        
        Request IMU stream of device

        :returns: :py:class:`blickfeld_scanner.scanner.stream.imu` object
        """
        return stream.imu(self.create_connection(), as_numpy=as_numpy)
 
    @staticmethod
    def sync(devices, scan_pattern = None, target_frame_rate = None, max_time_difference = 0.1):
        """ Sync two or more devices to record frames at the same time.

        :param server: list of scanners to sync
        :type server: list of :py:class:`blickfeld_scanner.scanner.scanner`
        :param scan_pattern: scan pattern that the devices should use, if None is given, it will select the current scan pattern
        :type scan_pattern: scan pattern, see: :any:`protobuf_protocol`,
        :param target_frame_rate: Frame rate that the devices should run. Maximum frame rate is dependent on the scan pattern, if None is given the maximum possible frame rate will be used.
        :type target_frame_rate: float
        :param max_time_difference: Maximum time difference allowed.
        :type max_time_difference: float
        """
        max_frame_rate = None
        for device in devices:
            sp_device = device.fill_scan_pattern(scan_pattern) if scan_pattern is not None else device.get_scan_pattern()
            if max_frame_rate is None or sp_device.frame_rate.maximum < max_frame_rate:
                max_frame_rate = sp_device.frame_rate.maximum
        
        if target_frame_rate and target_frame_rate < max_frame_rate:
            frame_rate = target_frame_rate
        else:
            frame_rate = max_frame_rate
        
        log.info("Syncing to frame rate %0.2f Hz (maximum %0.2f Hz)" % (frame_rate, max_frame_rate))
        
        for device in devices:
            sp_device = device.fill_scan_pattern(scan_pattern) if scan_pattern is not None else device.get_scan_pattern()
            sp_device.frame_rate.target = frame_rate
            device.set_scan_pattern(sp_device)
        log.info("Sync triggered")
        
        timestamps = []
        total_diff = 0
        for device in devices:
            timestamps.append(device.get_device_timestamp())
        for i, ts in enumerate(timestamps[1:], 1):
            diff = abs(timestamps[0] - ts)
            total_diff = total_diff + diff
            if diff > max_time_difference:
                raise Exception("Device timestamps are not synced (%s: %fs, %s: %fs => diff %fs)." 
                    % (devices[i].hostname_or_ip, ts, devices[0].hostname_or_ip, timestamps[0], diff))
        log.info("Total time difference with serial requests %0.3fs" % (total_diff))

class protocol_exception(Exception):
    """ Protocol exception class for errors, see errors in :any:`protobuf_protocol`

    :param value: Exception that occured
    :type value: Protocol Exception, see: :any:`protobuf_protocol`
    """
    def __init__(self, value):
        self.error = value

    def __format(self):
        e_desc = self.description()
        for match in re.finditer(re.compile(r"(\{([a-z_]+)\:(\%\d*\.*\d*[fsdu])\})"), e_desc):
            template, key, fmt = match.groups()
            if self.__contained_error().HasField(key):
                e_desc = e_desc.replace(template, str(fmt % getattr(self.__contained_error(), key)))
        return self.name() + str(": (errno: {})").format(self.errno()) + "\n\t" + e_desc
        
    def __field(self):
        return self.error.DESCRIPTOR.fields_by_name[str(self.error.WhichOneof("error"))]

    def __contained_error(self):
        return getattr(self.error, self.error.WhichOneof("error"))
        
    def errno(self):
        """ Returns error number

        :return: Error number
        """
        return self.__field().number
    
    def name(self):
        """ Returns error name as in :any:`protobuf_protocol`

        :return: Error name
        """
        return self.__field().message_type.full_name
    
    def description(self):
        """ Returns error description as in :any:`protobuf_protocol`

        :return: Error description
        """
        return self.__field().message_type.GetOptions().Extensions[options_pb2.e_desc]
        
    def __str__(self):
        return self.__format()
   
class connection(object):
    """ Class to create a connection to the Blickfeld LiDAR device

    :param hostname_or_ip: hostname or IP to which the connection should be established
    :type hostname_or_ip: str
    :param port: port on which the connection should be established
    :type port: int
    :param timeout: Timeout in seconds until connection is aborted
    :type timeout: int
    :param key_and_cert: String containing a private key and certificate for SSL connection
    :type key_and_cert: str
    :param ssl_protocol: The protocol used for an SSL connection
    :type ssl_protocol: see ssl lib: `ssl.PROTOCOL_...`
    """
    def __init__(self, hostname_or_ip, port, timeout, key_and_cert=None, ssl_protocol=ssl.PROTOCOL_TLS):
        self._key_and_cert = key_and_cert
        self.has_ssl = self._key_and_cert != None
        if self.has_ssl:
            self._ssl_context = ssl.SSLContext(ssl_protocol)
            
            # Write cert in temporary file
            fd, key_and_cert_file = tempfile.mkstemp()
            with open(fd, 'w') as f:
                f.write(self._key_and_cert)
            try:
                self._ssl_context.load_cert_chain(key_and_cert_file)
                self._ssl_context.load_verify_locations(key_and_cert_file)
            except:
                os.unlink(key_and_cert_file)
                raise
            os.unlink(key_and_cert_file)
            
            self._ssl_context.verify_mode = ssl.CERT_NONE
            self.socket = self._ssl_context.wrap_socket(socket.create_connection((hostname_or_ip, port), timeout=timeout), server_hostname=hostname_or_ip)
        else:
            self.socket = socket.create_connection((hostname_or_ip, port), timeout=timeout)
         
    def __del__(self):
        self.close()

    def clone(self, other_hostname_or_ip=None, other_port=None, other_key_and_cert=None):
        """ Copies connection object.
        
        :param other_hostname_or_ip: a different hostname or IP to use for the clone
        :type other_hostname_or_ip: str
        :param other port: a different port to use for the clone
        :type other port: int
        :param other_key_and_cert: a different string containing a private key and certificate for the clone
        :type other_key_and_cert: str
        """
        (old_hostname_or_ip, old_port) = self.socket.getpeername()

        return connection(other_hostname_or_ip or old_hostname_or_ip, 
                          other_port or old_port,
                          other_key_and_cert or self._key_and_cert)
        
    def send_request(self, req):
        """ Send request to the device

        :param req: Request to the device
        :type req: Request, see: :any:`protobuf_protocol`
        :return: Response of the device, see: :any:`protobuf_protocol`
        """
        self.send(req)
        return self.recv()

    def send(self, req):
        """ Send request to the device

        :param req: Request to the device
        :type req: Request, see: :any:`protobuf_protocol`
        """
        data = req.SerializeToString()

        self.socket.send(struct.pack('<I', len(data)))
        self.socket.send(data)

    def recv(self):
        """ Receive response of the device

        :return: Response of the device, see: :any:`protobuf_protocol`
        """
        len = struct.unpack('<I', self.__recv_all(4))[0]
        data = self.__recv_all(len)
        recv = connection_pb2.Response()
        recv.ParseFromString(data)

        if recv.HasField('error'):
            raise protocol_exception(recv.error)

        return recv

    def __recv_all(self, n):
        data = b''
        while len(data) < n:
            try:
                packet = self.socket.recv(n - len(data))
            except socket.timeout:
                err = error_pb2.Error()
                err.connection_abort.details = "Timeout after %u seconds" % (self.socket.timeout)
                raise protocol_exception(err)
            if not packet:
                err = error_pb2.Error()
                err.connection_abort.details = "Receive failed"
                raise protocol_exception(err)
            data += packet
        return data

    def close(self):
        """ Close connection to the device
        """
        if hasattr(self, "socket"):
            self.socket.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/update/manifest.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2
from ..update import hardware_pb2 as blickfeld_dot_update_dot_hardware__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/update/manifest.proto',
  package='blickfeld.protocol.update',
  syntax='proto2',
  serialized_options=b'H\002',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1f\x62lickfeld/update/manifest.proto\x12\x19\x62lickfeld.protocol.update\x1a\x17\x62lickfeld/options.proto\x1a\x1f\x62lickfeld/update/hardware.proto\"\xa2\x03\n\x0cmanifest_msg\x12\x17\n\x0fmin_bus_version\x18\x01 \x01(\t\x12\x45\n\nimage_type\x18\x02 \x01(\x0e\x32\x31.blickfeld.protocol.update.manifest_msg.ImageType\x12W\n\x13\x63ompatible_hardware\x18\x03 \x01(\x0b\x32:.blickfeld.protocol.update.manifest_msg.CompatibleHardware\x12\x15\n\x06secure\x18\x04 \x01(\x08:\x05\x66\x61lse\x1a\x97\x01\n\x12\x43ompatibleHardware\x12;\n\x07variant\x18\x01 \x03(\x0e\x32*.blickfeld.protocol.update.HardwareVariant\x12\x0b\n\x03msb\x18\x02 \x03(\t\x12\x13\n\x0bscanner_big\x18\x03 \x03(\t\x12\x15\n\rscanner_small\x18\x04 \x03(\t\x12\x0b\n\x03hsd\x18\x05 \x03(\t\"(\n\tImageType\x12\x0f\n\x0bPARTITIONED\x10\x00\x12\n\n\x06SYSTEM\x10\x01\x42\x02H\x02'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_update_dot_hardware__pb2.DESCRIPTOR,])



_MANIFEST_MSG_IMAGETYPE = _descriptor.EnumDescriptor(
  name='ImageType',
  full_name='blickfeld.protocol.update.manifest_msg.ImageType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PARTITIONED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SYSTEM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=499,
  serialized_end=539,
)
_sym_db.RegisterEnumDescriptor(_MANIFEST_MSG_IMAGETYPE)


_MANIFEST_MSG_COMPATIBLEHARDWARE = _descriptor.Descriptor(
  name='CompatibleHardware',
  full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant', full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware.variant', index=0,
      number=1, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msb', full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware.msb', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scanner_big', full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware.scanner_big', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scanner_small', full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware.scanner_small', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hsd', full_name='blickfeld.protocol.update.manifest_msg.CompatibleHardware.hsd', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=346,
  serialized_end=497,
)

_MANIFEST_MSG = _descriptor.Descriptor(
  name='manifest_msg',
  full_name='blickfeld.protocol.update.manifest_msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_bus_version', full_name='blickfeld.protocol.update.manifest_msg.min_bus_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image_type', full_name='blickfeld.protocol.update.manifest_msg.image_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compatible_hardware', full_name='blickfeld.protocol.update.manifest_msg.compatible_hardware', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='secure', full_name='blickfeld.protocol.update.manifest_msg.secure', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MANIFEST_MSG_COMPATIBLEHARDWARE, ],
  enum_types=[
    _MANIFEST_MSG_IMAGETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=539,
)

_MANIFEST_MSG_COMPATIBLEHARDWARE.fields_by_name['variant'].enum_type = blickfeld_dot_update_dot_hardware__pb2._HARDWAREVARIANT
_MANIFEST_MSG_COMPATIBLEHARDWARE.containing_type = _MANIFEST_MSG
_MANIFEST_MSG.fields_by_name['image_type'].enum_type = _MANIFEST_MSG_IMAGETYPE
_MANIFEST_MSG.fields_by_name['compatible_hardware'].message_type = _MANIFEST_MSG_COMPATIBLEHARDWARE
_MANIFEST_MSG_IMAGETYPE.containing_type = _MANIFEST_MSG
DESCRIPTOR.message_types_by_name['manifest_msg'] = _MANIFEST_MSG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

manifest_msg = _reflection.GeneratedProtocolMessageType('manifest_msg', (_message.Message,), {

  'CompatibleHardware' : _reflection.GeneratedProtocolMessageType('CompatibleHardware', (_message.Message,), {
    'DESCRIPTOR' : _MANIFEST_MSG_COMPATIBLEHARDWARE,
    '__module__' : 'blickfeld.update.manifest_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.update.manifest_msg.CompatibleHardware)
    })
  ,
  'DESCRIPTOR' : _MANIFEST_MSG,
  '__module__' : 'blickfeld.update.manifest_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.update.manifest_msg)
  })
_sym_db.RegisterMessage(manifest_msg)
_sym_db.RegisterMessage(manifest_msg.CompatibleHardware)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/status/main.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2
from ..status import scanner_pb2 as blickfeld_dot_status_dot_scanner__pb2
from ..status import temperature_pb2 as blickfeld_dot_status_dot_temperature__pb2
from ..status import server_pb2 as blickfeld_dot_status_dot_server__pb2
from ..status import time_synchronization_pb2 as blickfeld_dot_status_dot_time__synchronization__pb2
from ..status import imu_pb2 as blickfeld_dot_status_dot_imu__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/status/main.proto',
  package='blickfeld.protocol',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x62lickfeld/status/main.proto\x12\x12\x62lickfeld.protocol\x1a\x17\x62lickfeld/options.proto\x1a\x1e\x62lickfeld/status/scanner.proto\x1a\"blickfeld/status/temperature.proto\x1a\x1d\x62lickfeld/status/server.proto\x1a+blickfeld/status/time_synchronization.proto\x1a\x1a\x62lickfeld/status/imu.proto\"\xc7\x02\n\x06Status\x12\x33\n\x07scanner\x18\x01 \x01(\x0b\x32\".blickfeld.protocol.status.Scanner\x12<\n\x0ctemperatures\x18\x02 \x03(\x0b\x32&.blickfeld.protocol.status.Temperature\x12\x31\n\x06server\x18\x05 \x01(\x0b\x32!.blickfeld.protocol.status.Server\x12\x31\n\x03imu\x18\x07 \x01(\x0b\x32\x1e.blickfeld.protocol.status.IMUB\x04\xb0\xb5\x18\x01\x12R\n\x14time_synchronization\x18\x08 \x01(\x0b\x32..blickfeld.protocol.status.TimeSynchronizationB\x04\xb0\xb5\x18\x01*\x04\x08\x03\x10\x04*\x04\x08\x04\x10\x05*\x04\x08\x06\x10\x07'
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,blickfeld_dot_status_dot_scanner__pb2.DESCRIPTOR,blickfeld_dot_status_dot_temperature__pb2.DESCRIPTOR,blickfeld_dot_status_dot_server__pb2.DESCRIPTOR,blickfeld_dot_status_dot_time__synchronization__pb2.DESCRIPTOR,blickfeld_dot_status_dot_imu__pb2.DESCRIPTOR,])




_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='blickfeld.protocol.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='scanner', full_name='blickfeld.protocol.Status.scanner', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='temperatures', full_name='blickfeld.protocol.Status.temperatures', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='server', full_name='blickfeld.protocol.Status.server', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='imu', full_name='blickfeld.protocol.Status.imu', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_synchronization', full_name='blickfeld.protocol.Status.time_synchronization', index=4,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\260\265\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=True,
  syntax='proto2',
  extension_ranges=[(3, 4), (4, 5), (6, 7), ],
  oneofs=[
  ],
  serialized_start=249,
  serialized_end=576,
)

_STATUS.fields_by_name['scanner'].message_type = blickfeld_dot_status_dot_scanner__pb2._SCANNER
_STATUS.fields_by_name['temperatures'].message_type = blickfeld_dot_status_dot_temperature__pb2._TEMPERATURE
_STATUS.fields_by_name['server'].message_type = blickfeld_dot_status_dot_server__pb2._SERVER
_STATUS.fields_by_name['imu'].message_type = blickfeld_dot_status_dot_imu__pb2._IMU
_STATUS.fields_by_name['time_synchronization'].message_type = blickfeld_dot_status_dot_time__synchronization__pb2._TIMESYNCHRONIZATION
DESCRIPTOR.message_types_by_name['Status'] = _STATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'blickfeld.status.main_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.Status)
  })
_sym_db.RegisterMessage(Status)


_STATUS.fields_by_name['imu']._options = None
_STATUS.fields_by_name['time_synchronization']._options = None
# @@protoc_insertion_point(module_scope)
